from .fastapi_server import EmbeddedAPIServer
from .fastapi_server import EmbeddedAPIServerLab as EmbeddedAPIService

from .handlers import create_handler_from_router
from .fastapi_util import default_handler
from .fastapi_util import default_router
